from __future__ import annotations

from typing import Annotated
from datetime import datetime
from uuid import UUID
from pydantic import AnyUrl, BaseModel, Field, RootModel

from asknews_sdk.dto.base import BaseSchema, Article, Entities


class RedditPerspective(BaseModel):
    sentiment: int
    relevant: bool
    summary: str


class RedditThread(BaseModel):
    author: str
    author_comment_karma: int
    author_link_karma: int
    body: str
    classification: list[str]
    comments: list[RedditComment]
    comments_count: int
    date: datetime
    entities: Entities
    id: UUID
    key_takeaways: list[str]
    keywords: list[str]
    sentiment: int
    subreddit_name: str
    subreddit_url: str
    summary: str
    title: str
    topic: str
    upvotes: int
    url: str


class RedditComment(BaseModel):
    author: str
    body: str
    date: datetime
    upvotes: int


class IntraClusterStatistics(BaseModel):
    cluster_articles_pct: float = {}
    cluster_countries_pct: float = {}
    cluster_domains_pct: float = {}
    cluster_languages_pct: float = {}
    cluster_probabilities: dict = {}


class StoryResponseUpdate(BaseModel):
    uuid: Annotated[UUID, Field(title='UUID of the story update')]
    articles: list[Article]
    n_articles: int
    entities: Entities
    headline: str
    story: str
    story_update_ts: int
    sources_urls: dict[str, int]
    languages_pct: dict[str, float]
    countries_pct: dict[str, float]
    key_takeaways: list[str]
    contradictions: list[str]
    continent: str
    people: list[str]
    locations: list[str]
    new_information: str
    image_url: AnyUrl
    url_safe_title: str
    story_uuid: UUID
    categories: list[str]
    image_prompt: str
    reddit_perspective: RedditPerspective
    reddit_threads: list[RedditThread]
    languages: dict[str, int]
    keywords: list[str]
    intra_cluster_statistics: IntraClusterStatistics
    silhouette_score: dict
    article_ids: list[UUID]
    countries: dict[str, int]

class StoryResponse(BaseSchema):
    uuid: Annotated[UUID, Field(title='UUID')]
    categories: Annotated[list[str], Field(title='Categories')]
    countries: Annotated[dict[str, int], Field(title='Countries')]
    countries_pct: Annotated[dict[str, float], Field(title='Countries percentage')]
    current_update_uuid: Annotated[str, Field(title='Current update UUID')]
    requested_update_uuid: Annotated[str, Field(title='Requested update UUID')]
    generate_image: Annotated[bool, Field(title='Generate image flag')]
    keywords: Annotated[list[str], Field(title='Keywords')]
    languages: Annotated[dict[str, int], Field(title='Languages')]
    languages_pct: Annotated[dict[str, float], Field(title='Languages percentage')]
    locations: Annotated[list[str], Field(title='Locations mentioned')]
    meta_type: Annotated[str, Field(title='Meta type')]
    n_articles: Annotated[list[int], Field(title='Number of articles')]
    n_updates: Annotated[int, Field(title='Number of updates')]
    people: Annotated[list[str], Field(title='People mentioned')]
    reddit_sentiment: Annotated[list[int], Field(title='Reddit sentiment')]
    reddit_sentiment_timestamps: Annotated[list[int], Field(title='Reddit sentiment timestamps')]
    rolling_sentiment: Annotated[list[float], Field(title='Rolling sentiment')]
    sentiment: Annotated[list[int], Field(title='Sentiment')]
    sources: Annotated[dict[str, int], Field(title='Sources')]
    sources_urls: Annotated[dict[str, int], Field(title='Sources URLs')]
    topic: Annotated[str, Field(title='Topic')]
    topics: Annotated[list[str], Field(title='Topics')]
    updates: Annotated[list[StoryResponseUpdate], Field(title='Updates')]
    updated_ts: Annotated[int, Field(title='Updated timestamp')]

class CondensedStoryResponse(BaseModel):
    uuid: Annotated[UUID, Field(title='UUID')]
    headline: Annotated[str, Field(title='Headline')]
    story: Annotated[str, Field(title='Story text')]
    image_url: Annotated[str, Field(title='Image URL')]
    url_safe_title: Annotated[str, Field(title='URL-safe title')]
    categories: Annotated[list[str], Field(title='Categories')]
    sources_urls: Annotated[dict[str, int], Field(title='Sources URLs')]
    countries_pct: Annotated[dict[str, float], Field(title='Countries percentage')]
    languages_pct: Annotated[dict[str, float], Field(title='Languages percentage')]
    sentiment: Annotated[list[int], Field(title='Sentiment')]
    rolling_sentiment: Annotated[list[float], Field(title='Rolling sentiment')]
    n_articles: Annotated[list[int], Field(title='Number of articles')]
    continent: Annotated[str, Field(title='Continent')]
    story_update_ts: Annotated[int, Field(title='Story update timestamp')]
    key_takeaways: Annotated[list[str], Field(title='Key takeaways')]
    contradictions: Annotated[list[str], Field(title='Contradictions')]
    people: Annotated[list[str], Field(title='People mentioned')]
    locations: Annotated[list[str], Field(title='Locations mentioned')]
    new_information: Annotated[str, Field(title='New information')]

class StoriesResponse(BaseSchema, RootModel[list[StoryResponse]]):
    root: Annotated[list[StoryResponse], Field(title='StoriesResponse')]

class CondensedStoriesResponse(BaseSchema, RootModel[list[CondensedStoryResponse]]):
    root: Annotated[list[CondensedStoryResponse], Field(title='CondensedStoriesResponse')]


class SourceReportItem(BaseModel):
    bson_date: Annotated[datetime, Field(title='Bson Date')]
    n_bucket: Annotated[int, Field(title='Number of Buckets')]
    n_selected: Annotated[int, Field(title='Number of Selected')]
    bucket_counts: Annotated[dict[str, int], Field(title='Bucket Counts')]
    selected_counts: Annotated[dict[str, int], Field(title='Selected Counts')]
    bucket_pct: Annotated[dict[str, float], Field(title='Bucket Percentage')]
    selected_pct: Annotated[dict[str, float], Field(title='Selected Percentage')]

class SourceReportResponse(BaseSchema, RootModel[list[SourceReportItem]]):
    root: Annotated[list[SourceReportItem], Field(title='SourceReportResponse')]
